/*
 * This demo shows natural texture parameterization on a NURBS surface using
 * the base stream toolkit.
 */


#include <BStream.h>
#include <stdio.h>

static void WriteObject(BBaseOpcodeHandler*, BStreamFileToolkit*, FILE*, char*r, int);
static void zebra_texture(unsigned const, unsigned const, unsigned char * const);

static float const pts[][3] = {
	{ 3.0f, 1.0f, 0.0f}, 
	{ 1.0f, 1.0f, 0.0f}, 	{ 0.0f, 2.0f, 0.0f}, 	{ -2.0f, 2.0f, 0.0f}, 
	{ -2.0f, 3.0f, 0.0f},	{ 3.0f, 1.0f, 1.0f}, 	{ 1.0f, 1.0f, 1.0f}, 
	{ 0.0f, 2.0f, 1.0f}, 	{ -2.0f, 2.0f, 1.0f},	{ -2.0f, 3.0f, 1.0f},
	{ 3.0f, -1.0f, 2.0f},	{ 1.0f, -1.0f, 2.0f},	{ 0.0f, 0.0f, 2.0f}, 
	{ -2.0f, 0.0f, 2.0f},	{ -2.0f, 1.0f, 2.0f},	{ 3.0f, -1.0f, 3.0f},
	{ 1.0f, -1.0f, 3.0f},	{ 0.0f, 0.0f, 3.0f}, 	{ -2.0f, 0.0f, 3.0f}, 
	{ -2.0f, 1.0f, 3.0f},	{ 3.0f, 0.0f, 4.0f}, 	{ 1.0f, 0.0f, 4.0f}, 
	{ 0.0f, -1.0f, 4.0f},	{ -2.0f, -1.0f, 4.0f},	{ -2.0f, 2.0f, 4.0f},
};
static float const weights[][5] = {
	{ 1, 1, 1, 1, 1 },
	{ 1, 0.5f, 0.5f, 0.5f, 1 },
	{ 1, 0.5f, 0.5f, 0.5f, 1 },
	{ 1, 0.5f, 0.5f, 0.5f, 1 },
	{ 1, 1, 1, 1, 1 },
};
static float const u_knots[] = { 0, 0, 0, 1, 2, 3, 4, 4, 4, };
static float const v_knots[] = { 0, 0, 0, 1, 2, 3, 4, 4, 4, };

static unsigned const width = 256;
static unsigned const height = 256;
static unsigned char pixel_data[height][width][3];
static char mybuffer[4096];

int main(){
	zebra_texture(width, height, (unsigned char* const)pixel_data);
	
    FILE *flptr = fopen( "temp.hsf", "wb");
	
	BStreamFileToolkit mytoolkit;
    mytoolkit.PrepareBuffer(mybuffer, sizeof(mybuffer));

    TK_Header header;
    WriteObject(&header, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));

	/* Create an image. */
	TK_Image image;
	image.SetName("zebra_image");
	image.SetFormat(TKO_Image_RGB);
	image.SetSize(width,height);
	image.SetBytes(sizeof(pixel_data), (char const*)pixel_data);
	WriteObject(&image, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));

	/* Set Visibility for images off, faces on. */
	TK_Visibility vis;
	vis.SetGeometry(TKO_Geo_Image | TKO_Geo_Face);
	vis.SetValue(TKO_Geo_Face);
	WriteObject(&vis, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));

	/* Create a texture, set the image about as texture image. */
	TK_Texture texture;
	texture.SetParameterSource(TKO_Texture_Param_Source_Natural_UV);
	texture.SetName("zebra_texture");
	texture.SetImage("zebra_image");
	texture.SetFlags(TKO_Texture_Param_Source);
	WriteObject(&texture, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));

	/* Set the color of geometry to the texture. */
	TK_Color color;
	color.SetDiffuseName("zebra_texture");
	color.SetGeometry(TKO_Geo_Face);
	WriteObject(&color, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));

	/* Create the geometry. */
	TK_NURBS_Surface nurbs;
	nurbs.SetSurface( 3, 3, 5, 5, (float const*)pts, (float const*)weights, u_knots, v_knots );
	WriteObject(&nurbs, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));

	TK_Terminator term( TKE_Termination );
	WriteObject(&term, &mytoolkit, flptr, mybuffer, sizeof(mybuffer));
	
	fwrite(mybuffer, sizeof(char), mytoolkit.CurrentBufferLength(), flptr);

	fclose(flptr);
	return 0;
}


static
void WriteObject(BBaseOpcodeHandler * handler,
				 BStreamFileToolkit * tk,
				 FILE * fp,
				 char * buffer,
				 int size) 
{
	
    TK_Status status;

    // ask the toolkit to write out the object; the toolkit will write out as much as
    // the current buffer can hold
    status = handler->Write(*tk);

    // if there is still more data left to write; we have to loop through 
    // generating and writing buffers of data until we're done
    while (status == TK_Pending) 
    {
        // due to compression, preparing a new buffer might immediately fill that buffer
        // so we need an extra check to possibly export that buffer and prepare a new buffer
        while (status == TK_Pending) 
        {
            // ask the toolkit how much of the buffer is currently filled
            // this will normally be the same as the buffer size that we passed in
            int count = tk->CurrentBufferLength();

            // write the current out to our file since this example deals with 
            // exporting a file; however, instead of writing to a file, the buffer of
            // data could be sent to an application data-structure, or transmitted
            // over a network ('streamed'), etc...
            fwrite(buffer, sizeof (char), count, fp);

            // tell the toolkit where it can resume writing data; in this case, we
            // can reuse the same buffer once we're done with it
            status = tk->PrepareBuffer(buffer, size);
        }

        // ask the toolkit to continue trying to write out the object's data
        status = handler->Write(*tk);
    }
}

static
void zebra_texture(unsigned const w,
				   unsigned const h,
				   unsigned char * const pixel_data)
{
	unsigned row, col;

	for(row = 0; row < h; ++row){
		for(col = 0; col < w; ++col){
			if((row+col) / 8 % 2){
				pixel_data[3*row*w+3*col+0] = 0xff;
				pixel_data[3*row*w+3*col+1] = 0xff;
				pixel_data[3*row*w+3*col+2] = 0xff;
			} else {
				pixel_data[3*row*w+3*col+0] = 0x00;
				pixel_data[3*row*w+3*col+1] = 0x00;
				pixel_data[3*row*w+3*col+2] = 0x00;
			}
		}
	}
}
